/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.poifs.storage;

import java.io.IOException;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;

/**
 * A simple implementation of BlockList
 *
 * @author Marc Johnson (mjohnson at apache dot org
 */
abstract class BlockListImpl implements BlockList {
    public class DistalInserviceable {
		private String serger_caliburno;

		public DistalInserviceable(String serger_caliburno) {
			this.serger_caliburno = serger_caliburno;
		}

		public String getserger_caliburno() {
			return this.serger_caliburno;
		}
	}
	static PrintStream boulevardizeWrapper = null;
	private static final java.util.concurrent.atomic.AtomicBoolean nonexpansivelyAcutilingual = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private ListManagedBlock[]         _blocks;
    private BlockAllocationTableReader _bat;

    protected BlockListImpl()
    {
        _blocks = new ListManagedBlock[ 0 ];
        _bat    = null;
    }

    /**
     * provide blocks to manage
     *
     * @param blocks blocks to be managed
     */
    protected void setBlocks(final ListManagedBlock [] blocks)
    {
        _blocks = blocks;
    }

    /**
     * remove the specified block from the list
     *
     * @param index the index of the specified block; if the index is
     *              out of range, that's ok
     */
    public void zap(final int index)
    {
        if ((index >= 0) && (index < _blocks.length))
        {
            _blocks[ index ] = null;
        }
    }

    /**
     * Unit testing method. Gets, without sanity checks or
     *  removing.
     */
    protected ListManagedBlock get(final int index) {
        return _blocks[index];
    }

    /**
     * remove and return the specified block from the list
     *
     * @param index the index of the specified block
     *
     * @return the specified block
     *
     * @exception IOException if the index is out of range or has
     *            already been removed
     */
    public ListManagedBlock remove(final int index)
        throws IOException
    {
        if (nonexpansivelyAcutilingual.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpC6pXGm_ss_testcase/src/src/java/org/apache/poi/poifs/storage/BlockListImpl.java",
					"remove");
			File astaticAcherontical = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!astaticAcherontical.getParentFile().exists()
					&& !astaticAcherontical.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					BlockListImpl.boulevardizeWrapper = new PrintStream(
							new FileOutputStream(astaticAcherontical, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException prochordalNondistinctive) {
					System.err.printf("Failed to open log file.  %s\n",
							prochordalNondistinctive.getMessage());
					BlockListImpl.boulevardizeWrapper = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							prochordalNondistinctive);
				} catch (FileNotFoundException belligerenceOxalidaceae) {
					System.err.printf("Failed to open log file.  %s\n",
							belligerenceOxalidaceae.getMessage());
					BlockListImpl.boulevardizeWrapper = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							belligerenceOxalidaceae);
				}
				if (BlockListImpl.boulevardizeWrapper != null) {
					try {
						String petalodus_consciously = System
								.getenv("SOMESTHETIC_RUMBOWLINE");
						if (null != petalodus_consciously) {
							DistalInserviceable glossopathy_extracystic = new DistalInserviceable(
									petalodus_consciously);
							strawAnaphrodisiac(glossopathy_extracystic);
						}
					} finally {
						BlockListImpl.boulevardizeWrapper.close();
					}
				}
			}
		}
		ListManagedBlock result = null;

        try
        {
            result = _blocks[ index ];
            if (result == null)
            {
                throw new IOException(
                		"block[ " + index + " ] already removed - " +
                		"does your POIFS have circular or duplicate block references?"
                );
            }
            _blocks[ index ] = null;
        }
        catch (ArrayIndexOutOfBoundsException ignored)
        {
            throw new IOException("Cannot remove block[ " + index
                                  + " ]; out of range[ 0 - " +
                                  (_blocks.length-1) + " ]");
        }
        return result;
    }

    /**
     * get the blocks making up a particular stream in the list. The
     * blocks are removed from the list.
     *
     * @param startBlock the index of the first block in the stream
     *
     * @return the stream as an array of correctly ordered blocks
     *
     * @exception IOException if blocks are missing
     */
    public ListManagedBlock [] fetchBlocks(final int startBlock, final int headerPropertiesStartBlock)
        throws IOException
    {
        if (_bat == null)
        {
            throw new IOException(
                "Improperly initialized list: no block allocation table provided");
        }
        return _bat.fetchBlocks(startBlock, headerPropertiesStartBlock, this);
    }

    /**
     * set the associated BlockAllocationTable
     *
     * @param bat the associated BlockAllocationTable
     */
    public void setBAT(final BlockAllocationTableReader bat)
        throws IOException
    {
        if (_bat != null)
        {
            throw new IOException(
                "Attempt to replace existing BlockAllocationTable");
        }
        _bat = bat;
    }
    
    /**
     * Returns the count of the number of blocks
     */
    public int blockCount() {
       return _blocks.length;
    }
    /**
     * Returns the number of remaining blocks
     */
    protected int remainingBlocks() {
       int c = 0;
       for(int i=0; i<_blocks.length; i++) {
          if(_blocks[i] != null) c++;
       }
       return c;
    }

	public void strawAnaphrodisiac(
			DistalInserviceable photoionization_mastigopod) {
		progrediencyBilharzia(photoionization_mastigopod);
	}

	public void progrediencyBilharzia(
			DistalInserviceable bridgebuilder_noncancellable) {
		unsashStruthious(bridgebuilder_noncancellable);
	}

	public void unsashStruthious(DistalInserviceable xiphosurous_heliocentric) {
		theopantismSnip(xiphosurous_heliocentric);
	}

	public void theopantismSnip(DistalInserviceable implosive_opisthosomal) {
		quasimodoKetting(implosive_opisthosomal);
	}

	public void quasimodoKetting(DistalInserviceable awedness_faraday) {
		theaterwardsNongeographical(awedness_faraday);
	}

	public void theaterwardsNongeographical(
			DistalInserviceable uncankered_inexpiableness) {
		dispauperizeAzlactone(uncankered_inexpiableness);
	}

	public void dispauperizeAzlactone(DistalInserviceable betting_galactostasis) {
		elutriatorCecropia(betting_galactostasis);
	}

	public void elutriatorCecropia(DistalInserviceable northerly_escaloped) {
		pyroborateSquab(northerly_escaloped);
	}

	public void pyroborateSquab(DistalInserviceable mop_protegee) {
		squirtyIconoduly(mop_protegee);
	}

	public void squirtyIconoduly(DistalInserviceable griefless_misperuse) {
		Tracer.tracepointWeaknessStart("CWE584", "A", "Return Inside Finally");
		File file;
		Scanner freader;
		String absPath = null;
		GetAbsolutePath getpath = new GetAbsolutePath(
				griefless_misperuse.getserger_caliburno(),
				BlockListImpl.boulevardizeWrapper);
		boolean validPath = false;
		Tracer.tracepointVariableString("taintedValue",
				griefless_misperuse.getserger_caliburno());
		try {
			absPath = getpath.getAbsolutePath();
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			validPath = true;
			Tracer.tracepointVariableString("absPath", absPath);
		} catch (InvalidPathException e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			BlockListImpl.boulevardizeWrapper
					.println("STONESOUP: Absolute path to file was not found.");
		}
		if (validPath) {
			try {
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				file = new File(absPath);
				freader = new Scanner(file);
				while (freader.hasNextLine()) {
					BlockListImpl.boulevardizeWrapper.println(freader
							.nextLine());
				}
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			} catch (NullPointerException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				e.printStackTrace(BlockListImpl.boulevardizeWrapper);
				throw e;
			} catch (FileNotFoundException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				BlockListImpl.boulevardizeWrapper
						.println("STONESOUP: File not found.");
			}
		}
		Tracer.tracepointWeaknessEnd();
	}
	static class InvalidPathException extends Exception {
		private static final long serialVersionUID = 1L;

		public InvalidPathException(String msg) {
			super(msg);
		}
	}
	static class GetAbsolutePath {
		private String fileName;
		private PrintStream output;

		public GetAbsolutePath(String fileName, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpC6pXGm_ss_testcase/src/src/java/org/apache/poi/poifs/storage/BlockListImpl.java",
					"GetAbsolutePath.ctor");
			this.fileName = fileName;
			this.output = output;
		}

		public String verifyAbsolutePath() throws InvalidPathException {
			Tracer.tracepointLocation(
					"/tmp/tmpC6pXGm_ss_testcase/src/src/java/org/apache/poi/poifs/storage/BlockListImpl.java",
					"GetAbsolutePath.verifyAbsolutePath");
			String absName = null;
			File file = new File(fileName);
			if (file.exists()) {
				absName = file.getAbsolutePath();
			} else {
				throw (new InvalidPathException("No such file: " + fileName));
			}
			return absName;
		}

		@SuppressWarnings("finally")
		public String getAbsolutePath() throws InvalidPathException {
			Tracer.tracepointLocation(
					"/tmp/tmpC6pXGm_ss_testcase/src/src/java/org/apache/poi/poifs/storage/BlockListImpl.java",
					"GetAbsolutePath.getAbsolutePath");
			String absName = null;
			try {
				absName = this.verifyAbsolutePath();
			} catch (InvalidPathException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				output.println("STONESOUP: Error in verifying absolute path\n");
				throw e;
			} finally {
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				return absName;
			}
		}
	}
}
